c Package: PINOCCHIO 1.1
c
c Cosmological tools
c Written by Pierluigi Monaco
c Copyright (C) 2001
c
c Input: 
c     common /cosmology/omega0,omlam,nsp,csp,s8,h
c       omega0 = density parameter
c       omlam  = cosmological constant
c       nsp    = spectral slope (see comments in function POWER)
c       csp    = normalization constant for the power spectrum
c       s8     = sigma_8
c       h      = H_0/100 km/s
c
c     common /smoothing_filter/filter
c       filter = 0:  Gaussian smoothing
c       filter = 1:  SKS smoothing
c       filter = 2:  top-hat smoothing
c
c     common /ps/deltac
c        deltac:  usual PS parameter
c
c Other common blocks:
c
c     common /grow_common/amat(1:100),bmat(1:100)        !,b2mat(1:100)
c     common /Lambda_common/rv(1:100),lamv(1:100)        !,lamv2(1:100)
c     common /dLambda_common/rv2(1:100),dldr(1:100)      !,dldr2(1:100)
c     common /Radius_common/lst(1:100),rst(1:100)        !,rst2(1:100)
c     common /about_Lambda/r

c functions:
c     omega(z)        density parameter
c     hubble(z)       Hubble parameter (km/s/Mpc)
c     hubble_Gyr(z)   Hubble parameter (Gyr^-1)
c     time(z)         cosmological time (Gyr)
c     scale(t)        scale factor as a function of t (Gyr)
c     grow(z)         growing mode (=1/(1+z) at early times)
c     inverse_grow(b) redshift corresponding to b(t)
c     fomega(z)       d log b / d log a (z)
c     power(fk)       power spectrum
c     aLambda(r)      linear variance (squared! at z=0) at radius r
c     aLambda_once(r)     "     "     (does not need Lambda_start)
c     dLambda(r)      dLambda/dr at radius r
c     Radius(v)       radius corresponding to the linear variance v (Mpc)
c     slope(r)        effective spectral index at scale r
c     aMass(v)        mass corresponding to a linear variance v (M_sun)
c     Variance(m)     linear variance corresponding to a mass m in M_sun
c     Size(m)         radius corresponding to a mass m in M_sun
c     aM_star(z)      typical collapsing mass at z (M_sun)
c     window(kr)      smoothing window, according to filter
c     dodl(v)         dOmega/dLambda according to different approximations
c     an_h(m,z)       mass function n(M;z) of halos (PS, ST & JN)
c
c subroutines:
c     normalization()          of the power spectrum to the wanted sigma_8
c     moments_cont(Rsm,spmom)  moments of the power spectrum
c     interpola(...)           cubic interpolation routine
c
c auxiliary subroutines:
c     scale_start     initializations for scale
c     grow_start      initialization for grow and inverse_grow
c     Lambda_start    intialization for aLambda,dLambda and Radius
c
c
c auxiliay functions:
c     grow_integrand
c     Lambda_integrand
c     moments_integrand
c     theta
c     safelog
c
c N.B.: ALL DISTANCES, TIMES AND MASSES ARE GIVEN OR REQUIRED WITH THE 
c       SPECIFIED VALUE OF h (NOT RESCALED TO h=1).
c
      function omega(z)
c
c Cosmological density parameter as a function of redshift
c
c DIMENSIONLESS
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      real omega,z
      if (omega0.eq.1..and.omlam.eq.0.) then
         omega=1.
      else if (omega0.lt.1..and.omlam.eq.0.) then
         omega=1./(1.+(1./omega0-1.)/(1.+z))
      else if (omega0.lt.1.and.abs(omlam+omega0-1.).lt.1.e-4) then
         omega=1./(1+(1./omega0-1.)/(1.+z)**3)
      else
         print *,'Wrong cosmology in function OMEGA!'
         print *,'Omega_0 = ',omega0
         print *,'Omega_Lambda = ',omlam
         stop
      endif
      return
      end
c
c
c
      function hubble(z)
c
c Hubble parameter as a function of redshift
c
c DIMENSION: km/s/Mpc
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      real hubble,z,omega
      external omega
      hubble=100.*h*(1.+z)**1.5*sqrt(omega0/omega(z))
      return
      end
c
c
c
      function hubble_Gyr(z)
c
c Hubble parameter as a function of redshift
c
c DIMENSION: Gyr^-1
c
      real hubble_Gyr,z
      hubble_Gyr=hubble(z)/977.76
      return
      end
c
c
c
      function time(z)
c
c Cosmological time as a function of redshift
c
c DIMENSION: Gyr
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      real time,z,time0,asinh,acosh
      external asinh,acosh
c
      time0=9.7776/h
      if (omega0.eq.1..and.omlam.eq.0.) then
         time=time0/1.5/(1.+z)**1.5
      else if (omega0.lt.1..and.omlam.eq.0.) then
         time=time0*omega0*.5*(1.-omega0)**(-1.5)*(-acosh((z*omega0-omega0+2.)
     $   /(z*omega0+omega0))+2.*sqrt((1.-omega0)*(omega0*z+1.))/omega0/(1.+z))
      else if (omega0.lt.1.and.abs(omlam+omega0-1).lt.1.e-4) then
         time=time0/1.5/sqrt(omlam)*asinh(sqrt(omlam/omega0)/(1.+z)**1.5)
      else
         print *,'Wrong cosmology in function TIME!'
         print *,'Omega_0 = ',omega0
         print *,'Omega_Lambda = ',omlam
         stop
      endif

      return
      end
c
c
c
      subroutine scale_start()
c
c Calculates the vectors necessary to evaluate scale(t)
c
      integer nbin
      parameter(nbin=100)
      real time_v(nbin),scale_v(nbin)
      common /scale_common/time_v,scale_v
      real amin,dalog,alog,z
      parameter(amin=-3.,dalog=3.5)
      external time
c
      do i=1,nbin
         alog=amin+(i-1.)*dalog/(nbin-1.)
         z=10.**(-alog)-1.
         time_v(i)=safelog(time(z))
         scale_v(i)=alog
      enddo
      return
      end
c
c
c
      function scale(t)
c
c Scale factor as a function of time
c
c DIMENSIONLESS
c
      integer nbin
      parameter(nbin=100)
      real time_v(nbin),scale_v(nbin)
      common /scale_common/time_v,scale_v
      real scale,t,tlog
c
      if (time_v(1).eq.time_v(nbin)) then
         print *,'Please run SCALE_START before calling function SCALE(t)'
         stop
      else
         tlog=safelog(t)
         call interpola(time_v,scale_v,nbin,tlog,scale)
         scale=10.**scale
         return
      endif
      end
c
c
c
      subroutine grow_start()
c
c Calculates the integral for grow(z) in a Lambda Universe
c
      implicit none
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer nbin,i
      parameter(nbin=100)
      real amat(nbin),bmat(nbin)                      !,b2mat(nbin)
      common /grow_common/amat,bmat                   !,b2mat
      real tt,hh,bb,grow_integrand,ttmin,dtt,grow,z,tau,safelog
      parameter (ttmin=-2.5,dtt=2.7)
      external grow_integrand
c
      if (omlam.gt.0.and.abs(omega0+omlam-1.).gt.1.e-4) then
         print *,'Wrong cosmology in function grow(z)!'
         print *,'Omega_0 = ',omega0
         print *,'Omega_Lambda = ',omlam
         stop
      endif
c
      if (omega0.eq.1.) then
         do i=1,nbin
            amat(i)=i
            bmat(i)=0.
         enddo
      else if (omlam.eq.0.) then
         do i=1,nbin
            z=float(nbin-i)/(nbin-1.)*60.
            tau=sqrt((1.+z)/(1./omega0-1.)+1.)
            grow=2.5*(1.+3.*(tau*tau-1.)*(1.+.5*tau*log((tau-1.)/(tau+1.))))/
     $           (1./omega0-1.)
            amat(i)=safelog(1./(1.+z))
            bmat(i)=safelog(grow)
         enddo
c normalized to b(z=0)=1
         call interpola(amat,bmat,nbin,0.,bb)
         do i=1,nbin
            bmat(i)=bmat(i)-bb
         enddo
      else
         do i=1,nbin
            tt=10.**(ttmin+(i-1.)*dtt/(nbin-1.))
            hh=1./tanh(tt)
            call integrate(grow_integrand,hh,max(100.,hh*100.),bb)
            amat(i)=(1./omega0-1.)**(-1./3.)*sinh(tt)**(2./3.)
            bmat(i)=hh*bb*5./3./(1./omega0-1.)**(1./3.)
            amat(i)=safelog(amat(i))
            bmat(i)=safelog(bmat(i))
         enddo
c normalized to b(z=0)=1
         call interpola(amat,bmat,nbin,0.,bb)
         do i=1,nbin
            bmat(i)=bmat(i)-bb
         enddo
      endif
      return
      end
c
c
c
      function grow_integrand(x)
c
c Integrand for grow_start
c
      real grow_integrand,x
      grow_integrand=1./(x*x*(x*x-1)**(.33333333333333))
      return
      end
c
c
c
      function grow(z)
c
c Linear growing mode, normalized as grow=1/(1+z) at high redshift
c
c DIMENSIONLESS
c
      implicit none
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer nbin
      parameter(nbin=100)
      real amat(nbin),bmat(nbin)
      common /grow_common/amat,bmat
      real grow,z,aa,bb,safelog
      external safelog
c
      if (omega0.eq.1.) then
         grow=1./(1.+z)
      else
         if (amat(1).eq.amat(nbin)) then
            print *,'Please run GROW_START before calling function GROW(z)'
            stop
         endif
         aa=safelog(1./(1.+z))
         call interpola(amat,bmat,nbin,aa,bb)
         grow=10.**bb
      endif
      return
      end
c
c
c
      function inverse_grow(b)
c
c Redshift corresponding to the growing mode b
c
c DIMENSIONLESS
c
      implicit none
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer nbin
      parameter(nbin=100)
      real amat(nbin),bmat(nbin)
      common /grow_common/amat,bmat
      real inverse_grow,aa,b,safelog
      external safelog
c
      if (omega0.eq.1.) then
         inverse_grow=1./b-1.
      else 
         if (amat(1).eq.amat(nbin)) then
            print *,'Please run GROW_START before calling function INVERSE_GROW(z)'
            stop
         endif
         call interpola(bmat,amat,nbin,safelog(b),aa)
         inverse_grow=10.**(-aa)-1.
      endif
      return
      end
c
c
c
      function fomega(z)
c
c Peebles' f(Omega) function, dlogb/dloga
c
c DIMENSIONLESS
c
      real fomega,z,dz,db
      dz=.02
      db=.5*(grow(z-dz)-grow(z+dz))/dz
      fomega=db*(1.+z)/grow(z)
      return
      end
c
c
c
      function power(fk)
c
c nsp = [-3,1]:  power-law power spectrum
c nsp >= 10.: Power spectrum according to Efstathiou, Bond & White (1992)   
c          (Gamma=nsp-10. if nsp>10)
c nsp <=-10.: Power spectrum according Bardeen et al. (1986)
c          (Gamma=-nsp-10. if nsp<-10)
c
c N.B.: fk IS SUPPOSED TO BE IN Mpc^-1
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      real pk,fk,gamma,power,q
      if (fk.eq.0.) then
         power=0.
         return
      endif
      if (nsp.ge.-3..and.nsp.le.1.) then
         if (fk.gt.5.e-4.and.fk.lt.5.e4) then
            pk=fk**nsp
         else
            pk=0.
         endif
      else if (nsp.ge.10.) then
         if (nsp.eq.10.) then
            gamma=omega0*h
         else
            gamma=nsp-10.
         endif
         q=fk/h/gamma
         pk=q/(1.+(6.4*q+(3.*q)**1.5+(1.7*q)**2)**1.13)**(2./1.13)
      else if (nsp.le.-10.) then
         if (nsp.eq.-10.) then
            gamma=omega0*h
         else
            gamma=-nsp-10.
         endif
         q=fk/h/gamma
         pk=log(1.+2.34*q)**2*(1.+3.89*q+(16.1*q)*(16.1*q)+(5.64*q)**3+
     $        (6.71*q)**4)**(-.5)/q
      else
         print *,'Wrong nsp in function POWER'
         stop
      endif
      power=pk*csp
      return
      end
c
c
c
      function aLambda_once(r)
c
c Linear variance on scale r, with smoothing as specified by FILTER;
c performs the integral each time
c r: Mpc
c
c DIMENSIONLESS
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      real out,limit,aLambda_once,r,r2,Lambda_integrand
      integer filter
      common /smoothing_filter/filter
      common /about_Lambda/r2
      external Lambda_integrand
c
      r2=r
      if (filter.eq.0) then
         limit=log(5./r)
      else if (filter.eq.1) then
         limit=log(1./r)
      else
         limit=log(10.9041/r)
      endif
      call integrate(Lambda_integrand,-6.9,limit,out)
      aLambda_once=out
      return
      end
c
c
c
      subroutine moments_cont(Rsm,spmom)
c
c Spectral moments of the power spectrum
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer filter,espo
      common /smoothing_filter/filter
      common /moments_common/Rsm2,espo
      real Rsm,limit,spmom(9),dlogk,moments_integrand,res,Rsm2
      parameter(dlogk=0.04)
      external moments_integrand
c
      Rsm2=Rsm
      if (filter.eq.0) then
         limit=5./Rsm
      else if (filter.eq.1) then
         limit=1./Rsm
      else
         limit=10.9041/Rsm
      endif
      do i=1,5
         espo=2*(i-1)
         call integrate(moments_integrand,-6.9,limit,res)
         spmom(i)=sqrt(res)
      enddo
      spmom(6)=spmom(4)*spmom(4)/spmom(3)/spmom(5)
      spmom(7)=spmom(3)*spmom(3)/spmom(1)/spmom(5)
      spmom(8)=spmom(2)*spmom(2)/spmom(1)/spmom(3)
      spmom(9)=sqrt(3.)*spmom(2)/spmom(3)
c
      return
      end
c
c
c
      function moments_integrand(logk)
      real logk,k,moments_integrand,Rsm
      integer espo
      common /moments_common/Rsm,espo
      k=exp(logk)
      moments_integrand=power(k)*window(k*Rsm)**2*k**(3+espo)
      return
      end
c
c
c
      subroutine Lambda_start()
c
c Calculates the integrals for the mass variance
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer nbin
      parameter(nbin=100)
      real rv(nbin),lamv(nbin),dldr(nbin)
      real lst(nbin),rst(nbin),rv2(nbin)
      real rmin,dr,aLambda_once
      common /Lambda_common/rv,lamv
      common /dLambda_common/rv2,dldr
      common /Radius_common/lst,rst
      parameter (rmin=-2.2,dr=.06)
      external aLambda_once
c
      do i=1,nbin
         rv(i)=(rmin+(i-.5)*dr) !-safelog(h)
         rv2(i)=rv(i)
         r=10.**rv(i)
         lamv(i)=safelog(aLambda_once(r))
      enddo
      do i=1,nbin
         if (i.eq.1) then
            dldr(i)=(lamv(i+1)-lamv(i))/(rv(i+1)-rv(i))
         else if (i.eq.nbin) then
            dldr(i)=(lamv(i)-lamv(i-1))/(rv(i)-rv(i-1))
         else
            dldr(i)=(lamv(i+1)-lamv(i-1))/(rv(i+1)-rv(i-1))
         endif
         rst(i)=rv(nbin-i+1)
         lst(i)=lamv(nbin-i+1)
      enddo
      return
      end
c
c
c
      function window(x)
c
c Window function
c
c     N.B:  x =: kr !!!
c
c     filter = 0: Gaussian smoothing
c     filter = 1: SKS smoothing
c     filter = 2: top-hat smoothing
c
c DIMENSIONLESS
c
      real window,x,theta
      integer filter
      common /smoothing_filter/filter
      external theta
c
      if (filter.eq.0) then
         window=exp(-x*x/2.)
      else if (filter.eq.1) then
         window=theta(1.-x)
      else if (filter.eq.2) then
         if (x.eq.0.) then
            window=1.
         else if (x.lt.10.9041) then
            window=3.*(sin(x)/x**3-cos(x)/x**2)
         else
            window=0.
         endif
      else
         print *,'Unknown window function'
         print *,'filter = ',filter
         stop
      endif
      return
      end
c
c
c
      function Lambda_integrand(logk)
c
c Integrand for evaluation of Lambda
c
      real Lambda_integrand,r,k,logk
      common/about_Lambda/r
      k=exp(logk)
      if (k.lt.1.e3) then
         Lambda_integrand=power(k)*window(k*r)**2*k**3
      else
         Lambda_integrand=0.
      endif
      return
      end
c
c
c
      function aLambda(r)
c
c Linear variance on scale r, with smoothing as specified by Lambda_start
c r: Mpc 
c
c DIMENSIONLESS
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer nbin,warning
      parameter(nbin=100)
      real rv(nbin),lamv(nbin),r,aLambda,rlog,aLambda_once
      common /Lambda_common/rv,lamv
      save warning
c
      if (rv(1).eq.rv(nbin)) then
         aLambda=aLambda_once(r)
         if (warning.eq.0) then
            print *,'Warning: LAMBDA used before running LAMBDA_START'
            warning=1
         endif
      else
         rlog=safelog(r)
         call interpola(rv,lamv,nbin,rlog,aLambda)
         aLambda=10.**aLambda
      endif
      return
      end
c
c
c
      function dLambda(r)
c
c d Log Lambda/d Log r on scale r, with smoothing as specified by Lambda_start
c r: Mpc
c
c DIMENSIONLESS
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer nbin
      parameter(nbin=100)
      real rv(nbin),dldr(nbin),r,dLambda,rlog
      common /dLambda_common/rv,dldr
c
      if (rv(1).eq.rv(nbin)) then
         print *,'Please run LAMBDA_START before calling function DLAMBDA(r)'
         stop
      endif         
      rlog=safelog(r)
      call interpola(rv,dldr,nbin,rlog,dLambda)
      return
      end
c
c
c
      subroutine normalization()
c
c Normalization of the power spectrum to sigma_8
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer filter,filter_old,nbin
      common /smoothing_filter/filter
      parameter(nbin=100)
      real aLambda_once
      external aLambda_once
c
      filter_old=filter
      csp=1.
      filter=2
      csp=s8*s8/aLambda_once(8./h)
      filter=filter_old
      return
      end
c
c
c
      function Radius(v)
c
c Radius corresponding to variance v, with smoothing as specified 
c by Lambda_start
c
c DIMENSION: Mpc
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer nbin
      parameter(nbin=100)
      real rv(nbin),lamv(nbin),v,Radius,vlog
      common /Radius_common/rv,lamv
c
      if (rv(1).eq.rv(nbin)) then
         print *,'Please run LAMBDA_START before calling function RADIUS(r)'
         stop
      endif         
      vlog=safelog(v)
      call interpola(rv,lamv,nbin,vlog,Radius)
      Radius=10.**Radius
      return
      end
c
c
c
      function slope(r)
c
c Spectral slope at smoothing radius r
c r: Mpc
c
c DIMENSIONLESS
c
      real slope,r,dLambda
      external dLambda
      slope=-dLambda(r)-3.
      return
      end
c
c
c
      function aMass(v)
c
c Mass at linear variance V
c
c DIMENSION: M_sun
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      real aMass,v,r,Radius
      integer filter
      common /smoothing_filter/filter
      external Radius
c
      r=Radius(v)
      if (filter.eq.0) then
         aMass=(2*acos(-1.))**1.5*r**3
      else if (filter.eq.1) then
         aMass=6.*acos(-1.)**2*r**3
      else if (filter.eq.2) then
         aMass=4.*acos(-1.)*r**3/3.
      else
         print *,'Unknown smoothing in function AMASS'
         stop
      endif
      aMass=aMass*2.778e11*h*h*omega0
      return
      end
c
c
c
      function Variance(m)
c
c Variance corresponding to mass m
c m: M_sun
c
c DIMENSIONLESS
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      real Variance,m,r,aLambda
      integer filter
      common /smoothing_filter/filter
      external aLambda
c
      if (filter.eq.0) then
         r=(m/(2*acos(-1.))**1.5/(2.778e11*h*h*omega0))**(1./3.)
      else if (filter.eq.1) then
         r=(m/(6.*acos(-1.)**2*2.778e11*h*h*omega0))**(1./3.)
      else if (filter.eq.2) then
         r=(m/(4.*acos(-1.)*2.778e11*h*h*omega0/3.))**(1./3.)
      else
         print *,'Unknown smoothing in function VARIANCE'
         stop
      endif
      Variance=aLambda(r)
      return
      end
c
c
c
      function Size(m)
c
c Radius corresponding to mass m
c m: M_sun
c
c DIMENSION: Mpc
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      real Size,m
      integer filter
      common /smoothing_filter/filter
c
      if (filter.eq.0) then
         Size=(m/(2*acos(-1.))**1.5/(2.778e11*h*h*omega0))**(1./3.)
      else if (filter.eq.1) then
         Size=(m/(6.*acos(-1.)**2*2.778e11*h*h*omega0))**(1./3.)
      else if (filter.eq.2) then
         Size=(m/(4.*acos(-1.)*2.778e11*h*h*omega0/3.))**(1./3.)
      else
         print *,'Unknown smoothing in function SIZE'
         stop
      endif
      return
      end
c
c
c
      function aM_star(z)
c
c Typical collapsing mass at redshift z
c
c DIMENSION: M_sun
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      real deltac
      integer flag
      common /ps/deltac,flag
      real aM_star,z,aMass
      external aMass
      aM_star=aMass((deltac/grow(z))**2)
      return
      end
c
c
c
      function dodl(v)
c
c dOmega/dLambda
c
c input: common /ps/ deltac,flag
c
c     flag=0:  Press  & Schechter (1974)
c     flag=1:  Sheth & Tormen (2001)
c     flag=2:  Jenkins et al. (2001)
c
c DIFFERENTIALS: dLambda
c DIMENSIONLESS
c
      implicit none
      real deltac
      integer flag
      common /ps/deltac,flag
      real dodl,v,espo,sv,espo2,espo3,c
c      external erf
c
      sv=sqrt(v)
      c=0.39894228             ! c=1./sqrt(2.*acos(-1.))
      espo=deltac/sv
      if (flag.eq.0) then
         dodl=exp(-.5*espo**2)*espo*c
      else if (flag.eq.1) then
         espo2=0.84083292*espo
         dodl=0.3222*c*espo2*exp(-.5*espo2**2)*(1.+ 1./espo2**0.6)
      else if (flag.eq.2) then
         espo3=-log(sv)
         dodl=.5*0.315*exp(-abs(espo3+0.61)**3.8)
      else
         stop 'Unknown flag in function DODL'
      endif
c          
      return
      end
c
c
c
      FUNCTION erf(x)
c
c N.B.: DOES NOT REALLY COINCIDE WITH THE COMMONLY USED ERF...
c
      REAL erf,erfcc,x
      REAL t,z
      z=abs(x)
      t=1./(1.+0.5*z)
      erfcc=t*exp(-z*z-1.26551223+t*(1.00002368+t*(.37409196+t*
     *(.09678418+t*(-.18628806+t*(.27886807+t*(-1.13520398+t*
     *(1.48851587+t*(-.82215223+t*.17087277)))))))))
      if (x.lt.0.) erfcc=2.-erfcc
      erf=1.-erfcc/2.
      return
      END
c     
c
c
      function theta(x)
      real theta,x
      if (x.gt.0.) then
         theta=1.
      else
         theta=0.
      endif
      return
      end
c
c
c
      function asinh(x)
      real x,asinh
      asinh=log(x+sqrt(x*x+1.))
      return
      end
c
c
c
      function acosh(x)
      real x,acosh
      if (x.gt.1.) then
         acosh=log(x+sqrt(x*x-1.))
      else
         print *,'Undefined ACOSH function'
         acosh=0.
      endif
      return
      end
c
c
c
      function safelog(x)
      real x,safelog
      if (x.gt.0.) then
         safelog=log10(x)
      else
         safelog=-40.
      endif
      return
      end
c
c
c
      function sgn(x)
      real x,sgn
      if (x.lt.0.) then
         sgn=-1.
      else if (x.eq.0.) then
         sgn=0.
      else
         sgn=1.
      endif
c      if (x.ne.0.) then
c         sgn=x/abs(x)
c      else
c         sgn=0.
c      endif
      return
      end
c
c
c
      function an_h(mh,z)
c
c Number of halos of mass mh (M_sun) at redshift z
c
c N.B.: LINEAR OUTPUT
c
c DIMENSION: Mpc^{-3} M_sun^{-1}
c DIFFERENTIALS: dMh
c
c     inputs: common /cosmology/omega0,omlam,nsp,csp,s8,h
c             common /smoothing_filter/filter
c             common /ps/deltac,flag
c
c     Needs: Lambda_start() and grow_start() must have been run
c
      real nsp,csp,s8,h,omega0,omlam
      common /cosmology/omega0,omlam,nsp,csp,s8,h
      integer filter,flag
      common /smoothing_filter/filter
      real deltac
      common /ps/deltac,flag
      real an_h,mh,z,rho
      real Size
      external Size
c
      rho=2.778e11*h*h*omega0
      v=Variance(mh)*(grow(z))**2
      r=Size(mh)
c this was with the old dodl function
c      an_h=rho*dodl(v)*v*abs(dLambda(r)/3.)/mh/mh
c this is with the new dodl function
      an_h=rho*dodl(v)*abs(dLambda(r)/3.)/mh**2
      return
      end
c
c
c
      subroutine interpola(xv,yv,nv,x,y)
c
c Quadratic interpolation from x- and y-vectors
c
      integer nv
      real xv(nv),yv(nv),x,y,dy
      real xp(3),yp(3),xp4(4),yp4(4)
c
      if (x.le.xv(1)) then
         y=yv(1)
      else if (x.ge.xv(nv)) then
         y=yv(nv)
      else if (x.le.xv(2)) then
         do i=1,3
            xp(i)=xv(i)
            yp(i)=yv(i)
         enddo
         call polinomial(xp,yp,3,x,y,dy)
      else if (x.ge.xv(nv-1)) then
         do i=1,3
            xp(i)=xv(nv-3+i)
            yp(i)=yv(nv-3+i)
         enddo
         call polinomial(xp,yp,3,x,y,dy)
      else
         do iv=1,nv
            if (x.le.xv(iv)) goto 1
         enddo
 1       do i=1,4
            xp4(i)=xv(iv-3+i)
            yp4(i)=yv(iv-3+i)
         enddo
         call polinomial(xp4,yp4,4,x,y,dy)
      endif
      return
      end
